#name: tengine_client.sh
#Preconditions: Local source

Architecture=`lscpu |grep Architecture | awk '{print $2}'`

function TengineBuildStart(){
        #create the compile status directory
        StatusDir=/var/log/
        StatusFile=$StatusDir/compile.log
        if [ ! -d "$StatusDir" ]; then
            mkdir -p $StatusDir
        fi
        touch $StatusFile
        echo "compiler running" > $StatusFile
}

function TengineYumInstall(){
        #install dependency package
        yum install wget zip unzip tar coreutils  krb5-devel pcre pcre-devel sed rpm-build gcc gcc-c++  make libtool zlib zlib-devel perl* openssl*  -y  >> $StatusFile 2>&1
        if [ $? -ne 0 ];then
                echo "compiler error" >> $StatusFile
                echo "yum error!" | tee -a $StatusFile
                exit 1
        fi
}



function TengineSourceDownload(){
        #download openssl-1.1.1a source package
        if [ ! -f openssl-OpenSSL_1_1_1a.tar.gz ];then
                wget https://codeload.github.com/openssl/openssl/tar.gz/OpenSSL_1_1_1a --no-check-certificate --tries=3 --timeout=300  >> $StatusFile 2>&1
                if [ $? -ne 0 ];then
                        echo "compiler error" >> $StatusFile
                        echo "wget openssl code failed,begin to confirm if the openssl code are put in the folder! " | tee -a $StatusFile
                        exit 1
                else
                        mv OpenSSL_1_1_1a openssl-OpenSSL_1_1_1a.tar.gz

                        fi
        fi

        #download tengine
        if [ ! -f tengine-2.2.2.tar.gz ];then
                wget -N http://tengine.taobao.org/download/tengine-2.2.2.tar.gz --tries=3 --timeout=300  >> $StatusFile 2>&1
                if [ $? -ne 0 ];then
                        echo "compiler error" >> $StatusFile
                        echo "wget tengine code failed,begin to confirm if the tengine code are put in the folder! " | tee -a $StatusFile
                        exit 1
                fi
        fi
}

function TengineDepencyBuild(){
       if [ "$(printf '%s\n' "1.1.1a" "`openssl version  | awk '{print $2}'`" | sort -V | head -n1)" = "1.1.1a" ];then
             return 1;
       fi

        #install openssl-1.1.1a
        if [ ! -f openssl.spec ];then
                echo "compiler error" >> $StatusFile
                echo "confirm if the openssl.spec is put in the folder!" | tee -a $StatusFile
                exit 1
        fi

        #prepare to build openssl rpm packages
        mkdir -p /root/rpmbuild/SOURCES /root/rpmbuild/SPECS /root/rpmbuild/BUILD /root/rpmbuild/RPMS  /root/rpmbuild/BUILDROOT  /root/rpmbuild/SRPMS
        cp openssl-OpenSSL_1_1_1a.tar.gz /root/rpmbuild/SOURCES
        cp openssl.spec /root/rpmbuild/SPECS

        sed -i '/check_files/d' /usr/lib/rpm/macros
        sed -i '/unpackaged_files_terminate_build/d'  /usr/lib/rpm/macros

        #start to build the rpm package
        rpmbuild -bb /root/rpmbuild/SPECS/openssl.spec  >> $StatusFile 2>&1
        if [ $? -ne 0 ];then
                echo "compiler error" >> $StatusFile
                echo "rpmbuild openssl failed!" | tee -a $StatusFile
                exit 1
        fi
        ls /root/rpmbuild/RPMS/$Architecture/  >> $StatusFile 2>&1

        #install openssl-1.1.1a, needed by nginx
        rpm -ivh /root/rpmbuild/RPMS/$Architecture/openssl-libs-1.1.1a-3.el7.$Architecture.rpm /root/rpmbuild/RPMS/$Architecture/openssl-devel-1.1.1a-3.el7.$Architecture.rpm /root/rpmbuild/RPMS/$Architecture/openssl-1.1.1a-3.el7.$Architecture.rpm --force --nodeps >> $StatusFile 2>&1
        if [ $? -ne 0 ];then
                echo "compiler error" >> $StatusFile
                echo "deploy openssl failed!" | tee -a $StatusFile
                exit 1
        fi

        echo $PATH | grep "/usr/local/bin" > /dev/null
        if [ $? -ne 0 ];then
            echo export PATH=/usr/local/bin:\$PATH >> /etc/profile
            source /etc/profile
        fi

        cat /etc/ld.so.conf | grep "/usr/local/lib/" > /dev/null
        if [ $? -ne 0 ];then
            echo /usr/local/lib/  >> /etc/ld.so.conf
        fi

        cat /etc/ld.so.conf | grep "/usr/local/lib64/" > /dev/null
        if [ $? -ne 0 ];then
            echo /usr/local/lib64/  >> /etc/ld.so.conf
        fi

        ldconfig -v  >> $StatusFile 2>&1
        openssl version  >> $StatusFile 2>&1
}


function TengineRpmBuild(){
        if [ ! -f tengine.spec ];then
                echo "compiler error" >> $StatusFile
                echo "confirm if the tengine.spec is put in the folder! " | tee -a $StatusFile
                exit 1
        fi

        #prepare to build the rpm package
        cp tengine-2.2.2.tar.gz /root/rpmbuild/SOURCES
        cp tengine.spec /root/rpmbuild/SPECS

        #start to build the rpm package,the rpm path is /root/rpmbuild/RPMS/$Architecture/
        touch ~/.rpmmacros
        cat ~/.rpmmacros | grep "%debug_package %{nil}" > /dev/null
        rpmbuild -bb /root/rpmbuild/SPECS/tengine.spec >> $StatusFile 2>&1
        if [ $? -ne 0 ];then
                echo "compiler error" >> $StatusFile
                echo "rpmbuild tengine failed!" | tee -a $StatusFile
                exit 1
        fi

        ls /root/rpmbuild/RPMS/$Architecture/ >> $StatusFile 2>&1
}

function TengineBuildStop(){
        #copy rpm package to the specified path
        if [ ! -d "/kunpeng_autotest_share/packages/web/$msgId/$subUnit" ]; then
            mkdir -p /kunpeng_autotest_share/packages/web/$msgId/$subUnit
        fi

        mv /root/rpmbuild/RPMS/$Architecture/*.rpm .

        echo "compiler success" | tee -a $StatusFile
}

TengineBuildStart
TengineYumInstall
TengineSourceDownload
TengineDepencyBuild
TengineRpmBuild
TengineBuildStop
