#!/bin/bash

#
#    此脚本为tengine一键编译安装调优的入口脚本，需要配置参数后（稍后由参数说明），直接命令行运行sh install.sh即可实现tengine一键编译安装部署调优
#    tengine优化目前http长连接基于1node1个nginx实例，其余场景基于1P1个tengine实例，每个nginx实例需要1个http端口，1个https端口,一个index.html
#
#    关于源码相关的准备:
#        openssl下载路径：https://github.com/openssl/openssl/tree/OpenSSL_1_1_1a
#        tengine下载路径：http://tengine.taobao.org/download/tengine-2.2.2.tar.gz
#        将下载下来的openssl源码包openssl-OpenSSL_1_1_1a.zip和tengine源码包tengine-2.2.2.tar.gz放到  deps   目录下
#    
#    目录解释：
#        deps：编译源码的相关文件和脚本
#        scripts：调优相关脚本
#        install.sh：一键脚本
#        rollback.sh:回退所有操作脚本
#
#    需要手动配置的说明：
#        sense         --- 四种场景http长短连接，以及https长短连接：httpShort, httpLong, httpsShort, httpsLong,默认为 httpLong
#        eth0-eth3     --- 4个网口名，对应每个nginx实例
#                          其中https场景，只需要两个网口，即配置eth0与eth2即可
#        httpPort1-4   --- http端口
#        httpsPort1-4  --- https端口
#
#    Nginx调优说明：
#        关于中断绑核，因为绑核在重启后就会失效，因此会将绑核操作的脚本运行写进/etc/profile中，开机运行绑核动作        
#
#
##################需要配置的参数#####################
eth0=""                         #通信网口名,必填项
eth2=""                         #通信网口名,必填项
eth1=""                         #10GE通信网口名，http场景必填，其余不填
eth3=""                         #10GE通信tp场景必填，其余不填
sense="httpsShort"              #场景 httpsShort, httpsLong
httpPort1="8080"                #实例1的http端口
httpPort2="8081"                #实例2的http端口
httpsPort1="8443"               #实例1的https端口
httpsPort2="8444"               #实例2的https端口
#http长连接使用
httpPort3="8082"                #实例3的http端口，http长连接专用
httpPort4="8083"                #实例4的http端口，http长连接专用
httpsPort3="8445"               #实例3的https端口，http长连接专用
httpsPort4="8446"               #实例4的https端口，https长连接专用
#####################################################

#日志
log=tengine_install.log
rm -rf $log

#编译部署
function CompileAndDeploy(){
    #编译脚本执行
    cd ./deps/
    if [ ! -f tengine-2.2.2-1.el7_4.aarch64.rpm ];then
        sh tengine_build.sh
        if [ $? -ne 0 ];then
            echo "tengine build failed!"
            exit 1
        fi
    fi
    #安装脚本执行
    rpm -qa |grep tengine-2.2.2
    if [ $? -ne 0 ];then
        sh tengine_install.sh
        if [ $? -ne 0 ];then
            echo "tengine install failed !"
            exit 1
        fi
    fi
    cd ..
}
#nginx调优
function NginxOptimization(){
    cp startNginx.sh /usr/local/tengine-nginx/conf/startNginx.sh
    cp /usr/local/tengine-nginx/conf/nginx.conf /usr/local/tengine-nginx/conf/nginx.conf_1
    cp /usr/local/tengine-nginx/conf/nginx.conf /usr/local/tengine-nginx/conf/nginx.conf_2
    if [ $sense == "httpLong" ];then
        cp /usr/local/tengine-nginx/conf/nginx.conf /usr/local/tengine-nginx/conf/nginx.conf_3
        cp /usr/local/tengine-nginx/conf/nginx.conf /usr/local/tengine-nginx/conf/nginx.conf_4
    fi
    #将中断绑核脚本cp到nginx目录
    cp ./scripts/irq_default.sh /usr/local/tengine-nginx/conf/irq_default.sh
    systemctl start irqbalance.service
    systemctl stop irqbalance.service
    #nginx配置文件修改，中断绑核，并把中断绑核操作写入/etc/profile中
    \cp /etc/profile /etc/profile_bak
    if [ $sense == "httpsShort" ];then
        sh ./scripts/nginx_optimization.sh 0 48 $httpPort1 $httpsPort1 1
        sh ./scripts/irq_default.sh $eth0 8 40 47

        sh ./scripts/nginx_optimization.sh 48 48 $httpPort2 $httpsPort2 2
        sh ./scripts/irq_default.sh $eth2 7 89 95

    elif [ $sense == "httpsLong" ];then
        sh ./scripts/nginx_optimization.sh 0 41 $httpPort1 $httpsPort1 1
        sh ./scripts/irq_default.sh $eth0 7 41 47

        sh ./scripts/nginx_optimization.sh 48 40 $httpPort2 $httpsPort2 2
        sh ./scripts/irq_default.sh $eth2 6 89 95
    fi

}


#-----------------------------------shell beging------------------------------------
if [[ -z $eth0 ]] || [[ -z $eth2 ]] || [[ -z $sense ]];then
    echo "please config The NIC name in the install.sh"
    exit 1
fi
if [[ $sense == "httpLong" ]] && [[ -z $eth1 ]];then
    echo "please config The NIC name in the install.sh"
    exit 1
fi
if [[ $sense == "httpLong" ]] && [[ -z $eth3 ]];then
    echo "please config The NIC name in the install.sh"
    exit 1
fi

#编译安装
echo "################################## begine to build and install  ##################################" | tee -a $log
CompileAndDeploy | tee -a $log
#调优
echo "################################## begine to optimize ##################################" | tee -a $log
#sh ./scripts/gerneral_optimization.sh | tee -a $log
NginxOptimization | tee -a $log

echo "you need to reboot to make the optimization work" | tee -a $log
