#!/bin/bash

#nginxFile='./nginx.cnf'
coreStart=$1
coreCount=$2
httpPort=$3
httpsPort=$4
nginxFile="/usr/local/tengine-nginx/conf/nginx.conf_$5"
indexFile="/usr/local/tengine-nginx/html/index"
coreIndex="1"
affinity=""
echo $coreStart
echo $coreCount
echo $httpPort
echo $httpsPort
echo $nginxFile
echo "-------------------------"
#the function is to get the core index for worker_cpu_affinity
function GetCoreIndex(){
    count=$1
    if [ $count -eq 0 ];then
        return 1
    fi
    for ((i=0; i<$count;))
    do
        coreIndex=$coreIndex"0"
        let "i++"
    done
}
#the function is to get the value for worker_cpu_affinity
function GetWorkPorcess(){
    GetCoreIndex $coreStart
    affinity=$affinity' '$coreIndex

    for ((j=0; j<$coreCount-1; ))
    do
            GetCoreIndex 1
            affinity=$affinity' '$coreIndex
            let "j++"
    done

}
#change worker_processes and worker_cpu_affinity to set worker number and core bind
function ChangeCoreBind(){
    #change worker_processes
    sed -i "/worker_processes/c worker_processes $coreCount;" $nginxFile

    #change worker_cpu_affinity
    GetWorkPorcess
    line=`grep -c worker_cpu_affinity $nginxFile`
    if [ $line -eq 1 ];then  # if there is one worker_cpu_affinity,change it
        sed -i "/worker_cpu_affinity/c worker_cpu_affinity$affinity;" $nginxFile
    else                     # if there is no worker_cpu_affinity,add it after worker_processes
        sed -i "/worker_processes/a  worker_cpu_affinity$affinity;" $nginxFile
    fi
}

#change or add config
#input: 1 config name you want to change; 
#       2 the string you want the config change to
#       3 the postion where you can find the config(before or after the config)
#       4 "before" or "after":put the config before or after the position
#    PS: input of 3 and 4 will be use when the config is not exist and you need to add it,
#           so you need to know where you can put the config
function ChangeConfig(){
    config=$1
    change=$2
    position=$3
    beforeOrAfter=$4
    line=`grep -c $config $nginxFile`
    if [ $line -eq 1 ];then  # if there is the config,change it
        sed -i "/$config/c $change" $nginxFile
    else                     # if there is no config,add it before or after position
        if [ $beforeOrAfter == "after" ];then
            sed -i "/$position/a $change" $nginxFile
        else
            sed -i "/$position/i $change" $nginxFile
        fi
    fi
}

#add config for event
function AddEventConfig(){
    #change worker_connections
    sed -i "/worker_connections/c worker_connections 102400;" $nginxFile

    ChangeConfig 'worker_rlimit_nofile' 'worker_rlimit_nofile 102400;' 'events' 'before'
    ChangeConfig 'multi_accept' 'multi_accept on;' 'events' 'after'
    ChangeConfig 'accept_mutex' 'accept_mutex off;' 'events' 'after'
    ChangeConfig 'epoll' 'use epoll;' 'events' 'after'
}

#add config for http
function AddHttpConfig(){
    ChangeConfig 'tcp_nopush' 'tcp_nopush    on;' 'http {' 'after'
    ChangeConfig 'tcp_nodelay' 'tcp_nodelay    on;' 'http {' 'after'
    ChangeConfig 'sendfile_max_chunk' 'sendfile_max_chunk    512k;' 'http {' 'after'
    ChangeConfig 'keepalive_requests' 'keepalive_requests    2000;' 'http {' 'after'
    ChangeConfig 'client_header_buffer_size' 'client_header_buffer_size    512k;' 'http {' 'after'
    ChangeConfig 'large_client_header_buffers' 'large_client_header_buffers    4 512k;' 'http {' 'after'
    ChangeConfig 'server_names_hash_bucket_size' 'server_names_hash_bucket_size    128;' 'http {' 'after'
    ChangeConfig 'client_max_body_size' 'client_max_body_size    100m;' 'http {' 'after'
    ChangeConfig 'open_file_cache_errors' 'open_file_cache_errors    on;' 'http {' 'after'
    ChangeConfig 'open_file_cache_min_uses' 'open_file_cache_min_uses    1;' 'http {' 'after'
    ChangeConfig 'open_file_cache_valid' 'open_file_cache_valid    50s;' 'http {' 'after'
    sed -i "/http {/a open_file_cache max=102400 inactive=40s;" $nginxFile
    sed -i "/http {/a access_log /dev/null;" $nginxFile
    sed -i "/http {/a\check_shm_size 50m;" $nginxFile
}
# add reuseport in http listen
function AddReuseportAndChangeHttpPort(){
    line=`grep -n listen $nginxFile | awk -F ':' '{print $1}' | awk 'NR==1'`
    str=`head -n $line $nginxFile | tail -n 1`
    #cStr=${str%?}' reuseport;'
    sed -i "/$str/c listen $httpPort reuseport;" $nginxFile
}

function SetHttpsPort(){
#   sed -i "s/443 ssl/$httpsPort ssl/" $nginxFile
    sed -i  '$i  server {' $nginxFile
    sed -i  '$i  listen       '$httpsPort' ssl ;' $nginxFile
    sed -i  '$i  ssl_certificate      /usr/local/tengine-nginx/server_2048.crt;' $nginxFile
    sed -i  '$i  ssl_certificate_key  /usr/local/tengine-nginx/server_2048.key;' $nginxFile
    sed -i  '$i  ssl_session_cache    shared:SSL:1m;' $nginxFile
    sed -i  '$i  ssl_session_timeout  5m;' $nginxFile
    sed -i  '$i  ssl_protocols  TLSv1 TLSv1.1 TLSv1.2 TLSv1.3;' $nginxFile
    sed -i  '$i  ssl_ciphers  "EECDH+ECDSA+AESGCM EECDH+aRSA+AESGCM EECDH+ECDSA+SHA384 EECDH+ECDSA+SHA256 EECDH+aRSA+SHA384 EECDH+aRSA+SHA256 EECDH+aRSA+RC4 EECDH EDH+aRSA !aNULL !eNULL !LOW !3DES !MD5 !EXP !PSK !SRP !DSS !RC4";' $nginxFile
    sed -i  '$i  ssl_prefer_server_ciphers  on;' $nginxFile
    sed -i  '$i  location / {' $nginxFile
    sed -i  '$i   root   html;' $nginxFile
    sed -i  '$i  index  index.html index.htm;' $nginxFile
    sed -i  '$i  }' $nginxFile
    sed -i  '$i  }' $nginxFile
}
function ChangeIndexFile(){
    file='index'$1'.html'
    echo $file
    sed -i "s/index.html/$file/" $nginxFile
}

#---------------------------------------------------------------#
#                                                               #
#                           shell begin                         #
#                                                               #
#---------------------------------------------------------------#
if [ ! -f $nginxFile ];then
    echo "there is no nginx.conf!"
    exit
fi

ChangeCoreBind
AddEventConfig
AddHttpConfig
AddReuseportAndChangeHttpPort
SetHttpsPort
cp $indexFile'.html' $indexFile$5'.html'
ChangeIndexFile $5
